#!/usr/bin/ruby -Ke

# web board for hns
# Copyright (C) 2001 Yoichi Imai <yoichi@silver-forest.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

require 'kconv'
require 'config.rb'
require 'jcode'

HnsBBSVersion = '0.0.2'

class Article
  attr_accessor :delete_key, :msgid, :refid, :ref_diary, :time, :title, :name, :mail, :site, :addr, :body, :new_article
  def initialize(delete_key = "", msgid = "", refid = "", ref_diary = "", time = "", title = "", name = "", mail ="", site = "", addr = "", body = "")
    @delete_key = delete_key
    @msgid = msgid
    @refid = refid
    @ref_diary = ref_diary
    @title = title
    @name = name
    @mail = mail
    @site = site
    @addr = addr
    @body = body

    if time.kind_of?(Time) then
      @time = time
    elsif time == "" then
      @time = Time.now
    else
      @time = Time.at(time.to_i)
    end

    def @time.asctime
      (self + TimeGap).strftime('%y/%m/%d %H:%M')
    end
  end

  def to_s
    return [@delete_key, @msgid, @refid, @ref_diary, @time.to_i, @title, @name, @mail, @site, @addr, @body].join("\t")
  end

  def inspect
    '{' + [@delete_key, @msgid, @refid, @ref_diary, @time.asctime, @title, @name, @mail, @site, @addr, @body[0,30]].join(",") + '}'
  end

  def deleted?
     @delete_key == ''
  end

  def delete
      @delete_key = ''
      @body = ''
  end

  def can_delete?
    @delete_key != UnDeletableDeleteKey
  end

  def root?
    @refid == 'root'
  end

  def fresh?
    @time > Time.now - RecentTime
  end

  def fresh_mark
    if self.fresh? then
      return '<span class="new"> [NEW!]</span>'
    else
    return ''
    end
  end

  def display(msgid_hash = nil, refid_hash = nil, out = $defout)
    return if self.root? && @refid == ''

    out.puts('<div class="article">')

    if self.deleted? then
      out.puts('This message was deleted.<br>')
    elsif self.root? && @ref_diary != '' then
      out.puts("<h3>#{@title}</h3>")
    else
      out.puts('<div>')
      out.puts(%Q!<a href="#{WriteCGI}?refid=#{@msgid}">եĤ</a>!)
      out.puts(%Q! / <a href="#{DeleteCGI}?msgid=#{@msgid}"></a>!)
      out.puts('</div>')

      out.puts('<div class="header">')
      out.puts("Subject: #{@title}<br>")
      out.puts("Date: #{@time.asctime}<br>")
      if @mail == '' then
	out.puts("From: #{@name}<br>")
      else
	out.puts("From: #{@name} &lt;#{@mail}&gt;<br>")
      end
      out.puts(%Q!X-URL: <a href="#{@site}">#{@site}</a>!) unless @site == ''
      out.puts('</div>')
      
      out.puts("<p>#{@body}</p>")
    end
    
    if msgid_hash != nil && refid_hash != nil then
      out.puts('<div class="msglinks">')
      
      if self.root? == false then
	out.puts('<dl><dt>References:</dt>')
	out.puts('<dd>')
	out.puts(msgid_hash[@refid].get_subject(false))
	out.puts('</dd></dl>')
      end

      if refid_hash[@msgid] != nil then
	out.puts('<dl><dt>Follow-Ups:</dt>')
	refid_hash[@msgid].each do |article|
	  out.print('<dd>')
	  out.print(article.get_subject(false))
	  out.puts('</dd>')
	end
	out.puts('</dl>')
      end
     
      out.puts('</div>')
    end

    out.puts('</div>')
  end

  def get_subject(thr_mode = false)
    return_val = ''
    return_val << %Q(<span class="tlink"><a href="#{ReadCGI}?thr=on&amp;msgid=#{msgid}">[]</a></span>) if self.root? && thr_mode
    
    if @ref_diary != '' && self.root? then
      return_val << '<span class="dlink">' +
	            %Q(<a href="#{HnsPath}?to=#{@ref_diary}\##{@ref_diary}">) +
	            @title + '</a></span>'
    elsif self.deleted? then
      return_val << '[[[Deleted]]]'
    else
      return_val << '<span class="title">' +
	             %Q(<a href="#{ReadCGI}?msgid=#{@msgid}">) +
	             @title + '</a></span>'
      return_val << %Q(<span class="poster"> by #{@name}</span>)
      return_val << %Q(<span class="date">&nbsp;&nbsp;&nbsp;&nbsp;) +
	            @time.asctime +
	            self.fresh_mark +
	            '</span>'
    end
    return return_val
  end
  
end

def Article::new_with_kconv(delete_key = "", msgid = "", refid = "", ref_diary = "", time = "", title = "", name = "", mail = "", site = "", addr = "", body = "")
  return Article.new(delete_key,msgid,refid,ref_diary,time,title.toeuc,name.toeuc,mail.toeuc,site.toeuc,addr,body.toeuc)
end

class HnsBBS
  attr_reader :article_list, :msgid_hash, :refid_hash
  def initialize
    @msgid_hash = nil
    @refid_hash = nil
  end

  def load
    @article_list = []
    File.foreach(BBSData) do |line|
      line.chomp!
      raw = line.split("\t", 11)
      article = Article.new(*raw)
      @article_list.push(article)
    end
  end

  def save
    article_list = @article_list[0,MaxArticle]
    File.open(BBSData, "w") do |fp|
      article_list.each do |article|
	fp.puts(article)
      end
    end
    #File.utime(Time.now, Time.now, BBSData)
  end

  def add(article)
    @article_list.push(article)
  end

  def create_msgid_hash
    @msgid_hash = Hash.new
    @article_list.each do |article|
      @msgid_hash[article.msgid] = article
    end
  end

  def create_refid_hash
    @refid_hash = Hash.new
    @article_list.each do |article|
      @refid_hash[article.refid] ||= []
      @refid_hash[article.refid].push(article)
    end
  end
  
  def get_new_msgid
    new_id = 0
    @article_list.each do |article|
      new_id = article.msgid.to_i if new_id < article.msgid.to_i
    end
    return new_id + 1
  end
end

def HnsBBS::print_error(error, out = $defout)
  out.puts("Content-Type: text/html\n\n")
  out.print <<"END_OF_ERROR"
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
 "http://www.w3.org/TR/REC-html40/loose.dtd">
<html>
<head>
<title>web board for hns Error!</title>
</head>
<body>
<h1>Error!</h1>
#{error}
</body>
</html>
END_OF_ERROR
exit 0
end

def HnsBBS::html_print(out = $defout)
  out.puts("Last-Modified: #{CGI::rfc1123_date(File.mtime(BBSData))}")
  out.puts("Content-Type: text/html\n\n")
  out.puts <<"CENTER_OF_HEADER"
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
"http://www.w3.org/TR/REC-html40/loose.dtd">
<html lang="ja">
<head>
 <meta http-equiv="Content-Type" CONTENT="text/html; charset=euc-jp">
 <meta http-equiv="Content-Style-Type" content="text/css">
CENTER_OF_HEADER

  if BBSCSS != nil then
    out.puts(%Q(<link rel="STYLESHEET" type="text/css" href="#{BBSCSS}">))
  end

out.puts <<"END_OF_HEADER"
<title>#{BBSTitle}</title>
<style type="text/css">
<!--
p#copyright { text-align: right }
span.new { color: #ff0000; font-size: smaller }
span.date { font-size: smaller }
-->
</style>
</head>

<body>
<h2>#{BBSTitle}</h2>
<div align="center">
 <a href="../"></a> /
 <a href="index.cgi">ǥå</a> /
 <a href="write.cgi"></a> /
 <a href="read.cgi">ɽ</a> /
 <a href="admini.cgi"></a>
</div>
END_OF_HEADER

yield
#out.puts("#{yield}")

out.puts <<"END_OF_FOOTER"

<p id="copyright">
web board for hns(tentative name) version #{HnsBBSVersion}-ruby
</p>

</body>
</html>
END_OF_FOOTER

end

def HnsBBS::get_diary_title(diary)
  unless diary =~ /(\d{4})(\d{2})(\d{2})(\d+)/ then
      HnsBBS::print_error("Diary file open error")
  end

  year = $1; month = $2; day = $3; section = $4

  title = nil
  new_count = 1
  begin
    File.foreach("#{DiaryDir}/#{year}/d#{year}#{month}#{day}.hnf") do |line|
      if /^NEW (.*)/ =~ line then
	title = $1
      elsif /^LNEW [^ ]* (.*)/ =~ line then
	title = $1
      elsif /^RLNEW [^ ]* [^ ]* (.*)/ =~ line then
	title = $1
      end

      new_count -= 1 if /^GRP/ =~ line
      if /^L?R?NEW / =~ line then
	break if new_count == section.to_i
	new_count+=1
      end
    end
  rescue
    HnsBBS::print_error("Diary file open Error")
  end
  return title
end


class Cookie
end
def Cookie::get_cookie
  cookie = ENV['HTTP_COOKIE']
  return if cookie == nil
  ret = nil
  cookie.split(";").each do |pair|
    pair = pair.split("=")
    pair[0].tr!("\r\t\n ", "")
    if pair[0] == CookieName then
      ret = pair[1].tr("+", " ")
      break
    end
  end
  ret.split("|") if ret != nil
end

def Cookie::set_cookie(data)
  time = Time.now.gmtime + CookieExpire
  value = data.collect{|e| e.tr("|\n", "").tr(" ", "+")}.join("|")
  domain = "domain=#{CookieDomain}" if CookieDomain && CookieDomain != ''
  "Set-Cookie: #{CookieName}=#{value}; expires=#{CGI::rfc1123_date(time)}; #{domain}\n"
end

def html_convert(str, mode)
  ret = str.to_s.dup
  ret.gsub!(/\t/, '')
  ret.gsub!(%r!\&!, '&amp;')
  ret.gsub!(%r!<!, '&lt;')
  ret.gsub!(%r!>!, '&gt;')
  if mode == true then
    ret.gsub!(/[\r\n]*$/m, '')
    ret.gsub!(/(\r\n|\r|\n)/, '<br>')
    ret.gsub!(/ /, '&nbsp;')
  else
    ret.gsub!(/[\r\n]/, '')
  end
  return ret
end

